import $ from 'jquery';
import ElementElement from './Element';

require('selectize.js');
require('selectize.js/dist/css/selectize.css');
require('selectize.js/dist/css/selectize.default.css');

/**
 * <select> element.
 */
export default class Select extends ElementElement
{
    /**
     * Konstruktor.
     *
     * @param el
     */
    constructor(el)
    {
        super(el);

        this.$el.on('change', this.onChange.bind(this));

        if (this.$el.data('selectize')) {
            this.initSelectize();
        }
    }

    get value()
    {
        return this.selectize === undefined
            ? this.$el.val()
            : this.selectize.getValue();
    }

    set enabled(value)
    {
        super.enabled = value;

        if (this.selectize !== undefined) {
            if (value) {
                this.selectize.enable();
            } else {
                this.selectize.disable();
            }
        }
    }

    /**
     * Inicijalizira selectize.js na elementu.
     */
    initSelectize()
    {
        this.$el.removeClass('form-control');
        this.selectize = this.$el.selectize()[0].selectize;
    }

    setOptions(options)
    {
        this.enabled = false;

        if (this.$el.find('option:first-child').val() === '') {
            this.$el.find('option:not(:first-child)').remove();
        } else {
            this.$el.find('option').remove();
        }

        if (this.selectize !== undefined) {
            this.selectize.clearOptions();
        }

        $.each(options, function(value, label) {
            this.$el.append(
                $('<option/>')
                    .attr('value', value)
                    .html(label)
            );

            this.enabled = true;
        }.bind(this));

        if (this.selectize !== undefined) {
            $.each(this.$el.children('option'), function(i, option) {
                let $option = $(option);

                this.selectize.addOption({
                    value: $option.attr('value'),
                    text: $option.html(),
                });
            }.bind(this));
        }
    }

    onChange()
    {
        $(this).trigger('change');
    }
}