<?php
namespace Ep\Config;

/**
 * Class UniqueValueProcessor
 *
 * @package Ep\Config
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class UniqueValueProcessor
{
    /**
     * @param array $config
     *
     * @return array
     */
    public function __invoke(array $config): array
    {
        $this->removeDuplicates($config);

        return $config;
    }

    /**
     * @param array $config
     *
     * @return void
     */
    protected function removeDuplicates(array &$config): void
    {
        if (isset($config['unique_values'])) {
            $uniqueKeys = is_array($config['unique_values'])
                ? array_unique($config['unique_values'])
                : [$config['unique_values']];

            foreach ($uniqueKeys as $key) {
                $config[$key] = $this->arrayUnique($config[$key]);
            }
        }

        foreach ($config as $key => &$value) {
            if (is_array($value)) {
                $this->removeDuplicates($value);
            }
        }
    }

    /**
     * @param array $config
     *
     * @return array
     */
    protected function arrayUnique(array $config): array
    {
        $new = [];

        foreach ($config as $key => $value) {
            if (is_integer($key) && array_search($value, $new) !== false) {
                continue;
            }

            $new[$key] = $value;
        }

        return $new;
    }
}