<?php
namespace Ep\Currency\Adapter;

use Ep\Cache\AbstractCacheFactory;
use Ep\Http\AbstractHttpClientFactory;
use Interop\Container\ContainerInterface;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu CurrencyAdaptera.
 *
 * @package Ep\Currency\Adapter
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class AbstractAdapterFactory implements FactoryInterface
{
    /**
     * Stvara adapter.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return AdapterInterface
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config')['currency'];
        $adapterClass = $config['adapter'] ?? false;

        if (!$adapterClass || !class_exists($adapterClass)) {
            throw new \RuntimeException("Currency adapter nije definiran ili ne postoji.");
        }

        if (!defined(sprintf('%s::BASE_URI', $adapterClass))) {
            throw new \RuntimeException(sprintf("Currency adapter '%s' nema deklariranu konstantu 'BASE_URI'", $adapterClass));
        }

        $cacheFactory = new AbstractCacheFactory();
        $cache = $cacheFactory($container, '', [
            'adapter' => 'filesystem',
            'options' => [
                'namespace' => 'currency',
                'default_lifetime' => 3600,
            ],
        ]);

        $clientFactory = new AbstractHttpClientFactory();
        $client = $clientFactory($container, '', [
            'base_uri' => $adapterClass::BASE_URI,
            'cache' => $cache,
        ]);

        return new $adapterClass($client, $config['options'] ?? []);
    }
}