<?php
namespace Ep\Form\View\Helper;

use Ep\View\IconAwareInterface;
use Zend\Form\ElementInterface;
use Zend\Form\LabelAwareInterface;
use Zend\Form\View\Helper\FormButton as BaseFormButton;

/**
 * Renderira <button> element.
 *
 * @package Ep\Form\View\Helper
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class FormButton extends BaseFormButton
{
    /** @var FormIcon */
    protected $formIconHelper;

    protected function getFormIconHelper()
    {
        if (null === $this->formIconHelper) {
            $this->formIconHelper = method_exists($this->view, 'plugin')
                ? $this->view->plugin('form_icon')
                : new FormIcon();
        }

        return $this->formIconHelper;
    }

    /**
     * Renderira button.
     *
     * @param ElementInterface $element Element.
     *
     * @param null $buttonContent
     *
     * @return string
     */
	public function render(ElementInterface $element, $buttonContent = null)
	{
	    $classes = explode(' ', $element->getAttribute('class'));

	    if (false === array_search('btn', $classes)) {
	        array_unshift($classes, 'btn');
	        $element->setAttribute('class', implode(' ', $classes));
        }

        if (
            $element instanceof IconAwareInterface &&
            $element instanceof LabelAwareInterface &&
            strlen($element->getIcon()) > 0
        ) {
	        $formIconHelper = $this->getFormIconHelper();
	        $element->setLabelOption('disable_html_escape', true);
	        $buttonContent = $formIconHelper($element);
        }

        return parent::render($element, $buttonContent);
	}
}