<?php
namespace Ep\Http\Exception;

use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Throwable;

/**
 * Http odgovor ne udovoljava pravilima validacije.
 *
 * @package Ep\Http\Exception
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class InvalidResponseException extends \RuntimeException
{
    /** @var RequestInterface */
    protected $request;
    /** @var ResponseInterface */
    protected $response;

    /**
     * InvalidResponseException constructor.
     *
     * @param string $message
     * @param int $code
     * @param Throwable|null $previous
     * @param RequestInterface|null $request
     * @param ResponseInterface|null $response
     */
    public function __construct(
        string $message = "",
        int $code = 0,
        Throwable $previous = null,
        RequestInterface $request = null,
        ResponseInterface $response = null
    ) {
        parent::__construct($message, $code, $previous);

        $this->request = $request;
        $this->response = $response;
    }

    /**
     * Vraća upit koji je uzrokovao exception.
     *
     * @return RequestInterface
     */
    public function getRequest(): RequestInterface
    {
        return $this->request;
    }

    /**
     * Vraća odgovor koji ne prolazi validaciju.
     *
     * @return ResponseInterface
     */
    public function getResponse(): ResponseInterface
    {
        return $this->response;
    }
}