<?php
namespace Ep\Http\Validator;

use Ep\Http\Exception\InvalidResponseException;
use GuzzleHttp\Promise\Promise;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Zend\Validator\ValidatorChain as BaseValidatorChain;

/**
 * Validira HTTP odgovore.
 *
 * Ova klasa se ponaša kao ValidatorChain, ali može se koristiti i kao
 * Guzzle middleware.
 *
 * @package Ep\Http\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class ValidatorChain extends BaseValidatorChain
{
    /**
     * Validira response ili
     *
     * @param callable|mixed $handler
     *
     * @return bool|\Closure
     */
    public function __invoke($handler)
    {
        // koristimo se kao validator
        if (!is_callable($handler)) {
            return $this->isValid($handler);
        }

        // sad smo guzzle middleware
        return function(RequestInterface $request, array $options) use ($handler) {
            /** @var Promise $promise */
            $promise = $handler($request, $options);

            return $promise->then(
                function(ResponseInterface $response) use ($request) {
                    if (!$this->isValid($response)) {
                        throw new InvalidResponseException(
                            'HTTP odgovor nije prošao validaciju.',
                            0,
                            null,
                            $request,
                            $response
                        );
                    }

                    return $response;
                }
            );
        };
    }
}