<?php
namespace Ep\Middleware\ErrorHandler;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Zend\Log\LoggerInterface;

/**
 * Bilježi greške u log aplikacije.
 *
 * @package Ep\Middleware\ErrorHandler
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class LoggerListener
{
    /** @var LoggerInterface */
    private $logger;

    /**
     * Logger constructor.
     *
     * @param LoggerInterface $logger
     */
    public function __construct(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    /**
     * Bilježi grešku u log.
     *
     * @param \Exception|\Throwable $e
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     */
    public function __invoke($e, ServerRequestInterface $request, ResponseInterface $response)
    {
        $message = [sprintf('%s: %s', get_class($e), $e->getMessage())];
        $message[] = str_repeat('-', 80);
        $message[] = sprintf('Path: %s', $request->getUri()->getPath());
        $message[] = str_repeat('-', 80);
        $message[] = $e->getTraceAsString();
        $message[] = str_repeat('-', 80);

        $this->logger->emerg(implode(PHP_EOL, $message));
    }
}