<?php
namespace Ep\Middleware\ErrorHandler;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Zend\View\Renderer\RendererInterface;

/**
 * Renderira grešku kao template.
 *
 * @package Ep\Middleware\ErrorHandler
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class TemplatedResponse
{
    /** @var RendererInterface */
    private $renderer;

    /**
     * TemplatedResponseGenerator constructor.
     *
     * @param RendererInterface $renderer
     */
    public function __construct(RendererInterface $renderer)
    {
        $this->renderer = $renderer;
    }

    /**
     * Vraća renderirani template u tijelu odgovora.
     *
     * @param \Exception|\Throwable $e
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     *
     * @return ResponseInterface
     */
    public function __invoke($e, ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
    {
        $response = $response->withStatus(500);
        $body = $response->getBody();

        $rendered = $this->renderer->render('@core/error/500.html.twig');
        $body->write($rendered);

        return $response;
    }
}