<?php
namespace Ep\Stdlib;

use DateTime as BaseDateTime;
use DateTimeImmutable;

/**
 * Ova verzija DateTime klase podržava konverziju u OLE Automation Date.
 *
 * @package Ep\Stdlib
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-stdlib
 */
class DateTime extends BaseDateTime
{
    /** @var DateTimeImmutable */
    private $oleAutomationBaseDate;

    /**
     * Vraća OLE Automation vrijednost ovog DateTime objekta.
     *
     * @return float
     *
     * @throws \Exception
     */
    public function getOLEAutomationDate(): float
    {
        $baseDate = $this->getOLEAutomationBaseDate();
        $midnight = new DateTime($this->format('Y-m-d 00:00:00'));
        $dayInterval = $baseDate->diff($midnight);
        $msInterval = $this->diff($midnight);

        $days = $dayInterval->days;

        $msecs =
            ($msInterval->h * 3600000) +
            ($msInterval->i * 60000) +
            ($msInterval->s * 1000) +
            floor($this->format('u') / 100);

        $ole = $days + ($msecs / 86400000);

        if ($this->getTimestamp() < $this->getOLEAutomationBaseDate()->getTimestamp()) {
            $ole *= -1;
        }

        return $ole;
    }

    /**
     * Vraća točno vrijeme od kojeg se računa OLE datum.
     *
     * @return DateTimeImmutable
     *
     * @throws \Exception
     */
    private function getOLEAutomationBaseDate(): DateTimeImmutable
    {
        if (null === $this->oleAutomationBaseDate) {
            $this->oleAutomationBaseDate = new DateTimeImmutable('1899-12-30 00:00:00');
        }

        return $this->oleAutomationBaseDate;
    }
}