<?php
namespace Ep\Validator;

use Ep\Http\AbstractHttpClientFactory;
use Interop\Container\ContainerInterface;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu ReCaptcha validatora.
 *
 * @package Ep\Validator
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class ReCaptchaFactory implements FactoryInterface
{
    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return ReCaptcha
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $httpFactory = new AbstractHttpClientFactory();
        $http = $httpFactory($container, '', [
            'base_uri' => 'https://www.google.com/recaptcha/api/',
        ]);

        $config = $container->get('config')['google']['recaptcha'] ?? [];

        return new ReCaptcha($http, $config);
    }
}