<?php
namespace Ep\Validator;

use Zend\Validator\AbstractValidator;

/**
 * Validira adresu SMTP servera.
 *
 * @package Ep\Validator
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class SmtpServer extends AbstractValidator
{
	const INVALID = 'invalidHost';
	const NOT_SMTP = 'notSmtp';

	protected $messageTemplates = [
		self::INVALID => 'Ne mogu otvoriti vezu prema datom poslužitelju.',
		self::NOT_SMTP => 'Dati poslužitelj ne odgovara na SMTP zahtjeve.',
	];

    public function __construct($options = null)
    {
        parent::__construct($options);

        if (function_exists('_x')) {
            $this->messageTemplates = [
                self::INVALID => _x('Ne mogu otvoriti vezu prema datom poslužitelju.', 'smtp validator', 'ep'),
                self::NOT_SMTP => _x('Dati poslužitelj ne odgovara na SMTP zahtjeve.', 'smtp validator', 'ep'),
            ];
        }
    }

    /**
	 * Returns true if and only if $value meets the validation requirements
	 *
	 * If $value fails validation, then this method returns false, and
	 * getMessages() will return an array of messages that explain why the
	 * validation failed.
	 *
	 * @param  mixed $value
	 *
	 * @param null $context
	 *
	 * @return bool
	 */
	public function isValid($value, $context = null)
	{
		$port = 25;

		if (is_array($context) && isset($context['port']) && is_numeric($context['port'])) {
			$port = intval($context['port']);
		}

		$sock = @fsockopen($value, $port, $errno, $errstr, 1);

		if (false === $sock) {
			$this->error(self::INVALID);
			return false;
		}

		fwrite($sock, 'HELO ' . $value . "\r\n");

		$data = '';
		stream_set_timeout($sock, 1);

		while (is_resource($sock) && !feof($sock)) {
			$str = @fgets($sock, 515);

			$data .= $str;
			// If 4th character is a space, we are done reading, break the loop, micro-optimisation over strlen
			if ((isset($str[3]) and $str[3] == ' ')) {
				break;
			}

			// Timed-out? Log and break
			$info = stream_get_meta_data($sock);

			if ($info['timed_out']) {
				break;
			}
		}

		if (!preg_match('/^220/', $data)) {
			$this->error(self::NOT_SMTP);
			return false;
		}

		fclose($sock);
		return true;
	}
}