<?php
namespace Ep\View\Helper;

use Ep\Middleware\BaseUrl;
use Interop\Container\ContainerInterface;
use Zend\ServiceManager\Factory\FactoryInterface;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\View\Helper\BasePath;
use Zend\View\HelperPluginManager;

/**
 * Stvara novu instancu AssetUrl ViewHelpera.
 *
 * @package Ep\View\Helper
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class AssetUrlFactory implements FactoryInterface
{
    /**
     * Stvara ViewHelper.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return AssetUrl
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        /** @var BaseUrl $baseUrl */
        $baseUrl = $container->get('app.base_url');

        /** @var PluginManagerInterface $helpers */
        $helpers = $container->get(HelperPluginManager::class);
        /** @var BasePath $basePath */
        $basePath = $helpers->get('base_url');
        $basePath->setBasePath($baseUrl->getBaseUrl());

        $config = $container->get('config')['app'] ?? [];
        $options = [];

        if (isset($config['asset_manifest'])) {
            $options['manifest_path'] = $config['asset_manifest'];
        }

        if (isset($config['asset_url'])) {
            $options['public_path'] = $config['asset_url'];
        }

        return new AssetUrl($basePath, $options);
    }
}