<?php
namespace EpTest\Cache;

use Ep\Cache\AbstractCacheFactory;
use PHPUnit\Framework\TestCase;
use Psr\Cache\CacheItemPoolInterface;
use Psr\Container\ContainerInterface;
use Zend\ServiceManager\ServiceManager;

/**
 * Class AbstractCacheFactoryTest
 *
 * @package EpTest\Cache
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-framework
 */
class AbstractCacheFactoryTest extends TestCase
{
    /** @var ContainerInterface */
    protected $services;

    protected function setUp()
    {
        parent::setUp();

        $this->services = new ServiceManager([
            'abstract_factories' => [
                AbstractCacheFactory::class,
            ],
        ]);

        $this->services->setService('config', [
            'caches' => [
                'cache.apcu' => [
                    'adapter' => 'apcu',
                    'options' => [
                        'default_lifetime' => 12,
                    ],
                ],
                'cache.empty' => [],
                'cache.noadapter' => [
                    'options' => [],
                ],
            ],
        ]);
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testCanCreateAdapterFromConfig()
    {
        $this->assertTrue($this->services->has('cache.apcu'));

        $cache = $this->services->get('cache.apcu');

        $this->assertInstanceOf(CacheItemPoolInterface::class, $cache);
        $this->assertSame($cache, $this->services->get('cache.apcu'));
    }

    /**
     * @expectedException \Zend\ServiceManager\Exception\ServiceNotCreatedException
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testWillThrowExceptionWhenAdapterNotPresent()
    {
        $this->services->get('cache.noadapter');
    }
}