<?php
namespace EpTest\InputFilter;

use Ep\ConfigProvider;
use Ep\InputFilter\Factory;
use Ep\InputFilter\InputFilter;
use PHPUnit\Framework\TestCase;
use Zend\InputFilter\Input;
use Zend\InputFilter\InputFilterPluginManager;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\ServiceManager\ServiceManager;
use Zend\Validator\NotEmpty;
use Zend\Validator\ValidatorChain;

/**
 * Class InputFilterTest
 *
 * @package Ep\InputFilter
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-framework
 */
class InputFilterTest extends TestCase
{
    /**
     * Vraća novi input filter.
     *
     * @return InputFilter
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    protected function createInputFilter()
    {
        $config = (new ConfigProvider())();

        $container = new ServiceManager($config['dependencies']);
        $container->setService('config', $config);

        /** @var PluginManagerInterface $inputFilterManager */
        $inputFilterManager = $container->get(InputFilterPluginManager::class);

        return $inputFilterManager->get(InputFilter::class);
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testFactoryIsInjected()
    {
        $inputFilter = $this->createInputFilter();
        $factory = $inputFilter->getFactory();
        $this->assertInstanceOf(Factory::class, $factory);
    }

    /**
     * @expectedException \Zend\InputFilter\Exception\InvalidArgumentException
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testReplaceWillThrowExceptionForInvalidName()
    {
        $inputFilter = $this->createInputFilter();
        $inputFilter->replace(new Input(), 'invalid');
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testNewElementWillKeepTheOldOnesValidatorChain()
    {
        $inputFilter = $this->createInputFilter();
        $inputFilter->add([
            'name' => 'input',
            'validator_chain' => Mock\ValidatorChain::class,
        ]);

        $newChain = new ValidatorChain();
        $newChain->attach(new NotEmpty());

        $newInput = new Input();
        $newInput->setValidatorChain($newChain);

        $inputFilter->replace($newInput, 'input');
        $input = $inputFilter->get('input');

        $this->assertSame($newInput, $input);

        $validatorChain = $input->getValidatorChain();
        $this->assertInstanceOf(Mock\ValidatorChain::class, $validatorChain);

        $validators = $validatorChain->getValidators();
        $this->assertCount(1, $validators);

        $validator = $validators[0];
        $this->assertInstanceOf(NotEmpty::class, $validator['instance']);
    }
}