<?php
namespace EpTest\Middleware;

use Ep\Middleware\ForceWww;
use EpTest\ContainerAwareTrait;
use PHPUnit\Framework\TestCase;
use ReflectionClass;
use SplQueue;
use Zend\Stratigility\MiddlewarePipe;

/**
 * Class AppTest
 *
 * @package EpTest\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class AppTest extends TestCase
{
    use ContainerAwareTrait;

    /** @var string */
    protected $cwd;

    /**
     * Postavljanje testa.
     */
    protected function setUp()
    {
        parent::setUp();

        // spremamo trenutni dir za kasnije.
        $this->cwd = getcwd();
        chdir(__DIR__);
    }

    /**
     * Čišćenje nakon testa.
     */
    protected function tearDown()
    {
        parent::tearDown();

        // brišemo datoteke koje napravimo tijekom testova.
        $filesToDelete = [
            'var/log/app.log',
            'var/log',
            'var'
        ];

        foreach ($filesToDelete as $file) {
            if (is_dir($file)) {
                rmdir($file);
            } else if (file_exists($file)) {
                unlink($file);
            }
        }

        // vraćamo cwd gdje smo bili.
        chdir($this->cwd);
    }


    public function testAppLoggerWillCreateVarLog()
    {
        $container = $this->createContainer();
        /** @noinspection PhpUnhandledExceptionInspection */
        $container->get('app.logger');

        $this->assertTrue(file_exists('var/log/app.log'));
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     * @throws \ReflectionException
     */
    public function testMiddlwareIsInjectedIntoTheApplication()
    {
        $container = $this->createContainer([
            'app' => [
                'middleware' => [
                    'app.force_www',
                ],
            ],
        ]);

        /** @var MiddlewarePipe $app */
        $app = $container->get('app');

        $class = new ReflectionClass($app);
        $pipelineProperty = $class->getProperty('pipeline');
        $pipelineProperty->setAccessible(true);
        /** @var SplQueue $pipeline */
        $pipeline = $pipelineProperty->getValue($app);

        $found = false;
        foreach ($pipeline as $middleware) {
            if ($middleware instanceof ForceWww) {
                $found = true;
                break;
            }
        }

        $this->assertTrue($found);
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     * @throws \ReflectionException
     */
    public function testMiddlewareCanBeDisabledInConfigOverrides()
    {
        $container = $this->createContainer([
            'app' => [
                'middleware' => [
                    'app.force_www',
                    'app.force_www' => false,
                ],
            ],
        ]);

        /** @var MiddlewarePipe $app */
        $app = $container->get('app');

        $class = new ReflectionClass($app);
        $pipelineProperty = $class->getProperty('pipeline');
        $pipelineProperty->setAccessible(true);
        /** @var SplQueue $pipeline */
        $pipeline = $pipelineProperty->getValue($app);

        $found = false;
        foreach ($pipeline as $middleware) {
            if ($middleware instanceof ForceWww) {
                $found = true;
                break;
            }
        }

        $this->assertFalse($found);
    }
}