<?php
namespace EpTest\Middleware;

use Ep\Middleware\BaseUrl;
use EpTest\ContainerAwareTrait;
use PHPUnit\Framework\TestCase;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Zend\Diactoros\Response;
use Zend\Diactoros\ServerRequestFactory;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\Stratigility\MiddlewarePipe;

use function Zend\Stratigility\middleware;
use Zend\Stratigility\MiddlewarePipeInterface;
use Zend\View\Helper\BasePath;
use Zend\View\HelperPluginManager;

/**
 * Class BaseUrlTest
 *
 * @package EpTest\Middleware
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-framework
 */
class BaseUrlTest extends TestCase
{
    use ContainerAwareTrait;

    public function testCanDetectBaseUrlAndPath()
    {
        $request = ServerRequestFactory::fromGlobals([
            'SCRIPT_FILENAME' => '/home/user/directory/index.php',
            'SCRIPT_NAME' => '/directory/index.php',
            'PHP_SELF' => '/directory/index.php',
            'REQUEST_URI' => '/directory/',
        ]);

        $pipeline = new MiddlewarePipe();
        $pipeline->pipe(new BaseUrl());

        $pipeline->pipe(middleware(function(ServerRequestInterface $request) {
            $this->assertEquals('/directory', $request->getAttribute(BaseUrl::BASE_URL));
            $this->assertEquals('/directory', $request->getAttribute(BaseUrl::BASE_PATH));
            return new Response();
        }));

        $pipeline->handle($request);
    }

    public function testCanDetectBaseUrlWithScriptNameInRequestUri()
    {
        $request = ServerRequestFactory::fromGlobals([
            'SCRIPT_FILENAME' => '/home/user/webshop/wp/wp-admin/install.php',
            'SCRIPT_NAME' => '/webshop/wp/wp-admin/install.php',
            'PHP_SELF' => '/webshop/wp/wp-admin/install.php',
            'REQUEST_URI' => '/webshop/wp/wp-admin/install.php',
        ]);

        $pipeline = new MiddlewarePipe();
        $pipeline->pipe(new BaseUrl());

        $pipeline->pipe(middleware(function(ServerRequestInterface $request) {
            $this->assertEquals('/webshop/wp/wp-admin', $request->getAttribute(BaseUrl::BASE_URL));
            $this->assertEquals('/webshop/wp/wp-admin', $request->getAttribute(BaseUrl::BASE_PATH));
            return new Response();
        }));

        $pipeline->handle($request);
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testWillInjectBaseUrlIntoBasePathViewHelper()
    {
        $container = $this->createContainer();
        /** @var MiddlewarePipeInterface $pipeline */
        $pipeline = new MiddlewarePipe();
        /** @var MiddlewareInterface $middleware */
        $middleware = $container->get('app.base_url');

        $pipeline->pipe($middleware);

        $oldServer = $_SERVER;

        $_SERVER = array_merge($_SERVER, [
            'SCRIPT_FILENAME' => '/home/user/directory/index.php',
            'SCRIPT_NAME' => '/directory/index.php',
            'PHP_SELF' => '/directory/index.php',
            'REQUEST_URI' => '/directory/',
        ]);

        $request = ServerRequestFactory::fromGlobals();

        $pipeline->pipe(middleware(function() use ($container) {
            /** @var PluginManagerInterface $helpers */
            $helpers = $container->get(HelperPluginManager::class);
            /** @var BasePath $basePath */
            $basePath = $helpers->get('base_url');

            $this->assertEquals('/directory/', $basePath(''));

            return new Response();
        }));

        $pipeline->handle($request);

        $_SERVER = $oldServer;
    }
}